library kisrs;

import 'package:flutter/material.dart';
import 'package:rxdart/rxdart.dart';
import 'dart:async';

/// DataPoint<T> is a wrapper for RxDart `BehaviorSubject`.
/// ### Remember to close the DataPoint to avoid leaking!
/// 
/// Exposes inWidget, a StreamBuilder wrapper.
@immutable
class DataPoint<T> {
  DataPoint({void Function(T) onChange})
      : _subject = BehaviorSubject<T>(),
        _onChange = onChange;
  DataPoint.withValue(T v, {void Function(T) onChange})
      : _subject = BehaviorSubject<T>.seeded(v),
        _onChange = onChange;

  final void Function(T) _onChange;
  final BehaviorSubject<T> _subject;
  Stream<T> get stream => _subject.stream;
  T get value => _subject.value;
  set value(T newValue) {
    _subject.sink.add(newValue);
    if(_onChange!=null){
      _onChange(newValue);
    }
  }

  Future<dynamic> close() => _subject.close();
  String toString() => value.toString();

  Widget inWidget(Widget Function(BuildContext context, T data) builder) {
    return StreamBuilder<T>(
      initialData: value,
      stream: stream,
      builder: (BuildContext context, AsyncSnapshot<T> snapshot) {
        return builder(context, snapshot.data);
      },
    );
  }
}

/// Store<T> is an InheritedWidget that stores data.
///
/// If you need to do cleanup when the Widget is being disposed,
/// use a StoreManager<T> instead.
///
/// Usage example:
/// `Store<String>(child: MyWidget(), data: "A string");`
///
/// Then, when you want to get the data associated with the store:
/// `Store.of<String>(context);`
///
/// You can have multiple Stores in one widget tree, but you can only access the latest
/// Store of a given Type.
@immutable
class Store<T> extends InheritedWidget {
  Store({@required Widget child, @required this.data}) : super(child: child);
  final T data;
  @override
  bool updateShouldNotify(InheritedWidget old) => true;
  static Type _typeOf<T>() => T;
  static T of<T>(BuildContext context) =>
      (context.inheritFromWidgetOfExactType(_typeOf<Store<T>>()) as Store<T>)
          .data;
}

/// StoreManager contains a `Store` and calls `data.dispose()` when you dispose it.
/// This is useful when you want to clean up some stuff (like Streams)
@immutable
class StoreManager<T> extends StatefulWidget {
  StoreManager({@required this.child, @required this.data});
  final T data;
  final Widget child;
  @override
  _StoreManagerState<T> createState() => _StoreManagerState<T>();
}

class _StoreManagerState<T> extends State<StoreManager> {
  @override
  Widget build(BuildContext context) {
    return Store<T>(child: widget.child, data: widget.data);
  }

  void dispose() {
    if (widget.data.dispose) {
      widget.data.dispose();
    }
    super.dispose();
  }
}
